//
// $Id: FileOutStream.m,v 1.10 2005/06/15 11:43:47 johnstrr Exp $
//
// vi: set ft=objc:

/*
 * ObjectiveLib - a library of containers and algorithms for Objective-C
 *
 * Copyright (c) 2004
 * Will Mason
 *
 * Portions:
 *
 * Copyright (c) 1994
 * Hewlett-Packard Company
 *
 * Copyright (c) 1996,1997
 * Silicon Graphics Computer Systems, Inc.
 *
 * Copyright (c) 1997
 * Moscow Center for SPARC Technology
 *
 * Copyright (c) 1999 
 * Boris Fomitchev
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You may contact the author at will_mason@users.sourceforge.net.
 */

#import "FileOutStream.h"
#import "Macros.h"
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSFileHandle.h>
#import <Foundation/NSException.h>
#import <errno.h>
#import <fcntl.h>
#import <sys/stat.h>
#import <string.h>
#import <unistd.h>
#import <limits.h>

#ifdef WIN32
#define fsync _commit
#define S_IRGRP 0
#define S_IROTH 0
#endif

@interface OLFileOutStream (ProtectedMethods)

- (ssize_t) writeImpl: (int)fdesc buffer: (const uint8_t*)buffer count: (unsigned)count;

@end

@implementation OLFileOutStream

+ (id) streamWithFileHandle: (NSFileHandle*)handle
{
    OL_BEGIN_AUTO_CTOR(OLFileOutStream)
        initWithFileHandle: handle
    OL_END_AUTO_CTOR;
}

+ (id) streamWithPath: (NSString*)path
{
    OL_BEGIN_AUTO_CTOR(OLFileOutStream)
        initWithPath: path
    OL_END_AUTO_CTOR;
}

- (id) initWithFD: (int)fileDesc
{
    [super init];
    fd = fileDesc;
    return self;
}

- (id) initWithFileHandle: (NSFileHandle*)handle
{
    return [self initWithFD: [handle fileDescriptor]];
}

- (id) initWithPath: (NSString*)path
{
    return [self initWithPath: path append: NO];
}

- (id) initWithPath: (NSString*)path append: (BOOL)append
{
    char nameBuf[PATH_MAX + 1];
    int flags = O_WRONLY | O_CREAT;
    mode_t mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;

    [super init];
    flags |= (append ? O_APPEND : O_TRUNC);
    if (![path getFileSystemRepresentation: nameBuf maxLength: PATH_MAX])
    {
        [NSException raise: OLInputOutputException
            format: @"Cannot convert \"%@\" into a file name", path];
    }
    fd = open(nameBuf, flags, mode);
    if (fd == -1)
    {
        [NSException raise: OLInputOutputException
            format: @"Error opening file - %s", strerror(errno)];
    }
    return self;
}

- (void) dealloc
{
    [self close];
    [super dealloc];
}

- (void) close
{
    if (!(fd == -1 || fd == STDOUT_FILENO || fd == STDERR_FILENO))
    {
        if (close(fd) != 0)
        {
            fd = -1;
            [NSException raise: OLInputOutputException
                format: @"Error closing file - %s", strerror(errno)];
        }
    }
    fd = -1;
}

- (int) fileDescriptor
{
    return fd;
}

- (void) flush
{
    if (fd != -1)
    {
        if (fsync(fd) != 0)
        {
            [NSException raise: OLInputOutputException
                format: @"Error syncing file - %s", strerror(errno)];
        }
    }
}

- (unsigned) writeBytes: (const uint8_t*)bytes count: (unsigned)count
{
    ssize_t written = [self writeImpl: fd buffer: bytes count: count];

    if (written == -1)
    {
        [NSException raise: OLInputOutputException
            format: @"Error writing to file - %s", strerror(errno)];
    }
    return written;
}

- (ssize_t) writeImpl: (int)fdesc buffer: (const uint8_t*)buffer count: (unsigned)count
{
    return write(fdesc, buffer, count);
}

@end
