//
// $Id: HashSet.m,v 1.13 2005/05/18 06:46:53 will_mason Exp $
//
// vi: set ft=objc:

/*
 * ObjectiveLib - a library of containers and algorithms for Objective-C
 *
 * Copyright (c) 2004
 * Will Mason
 *
 * Portions:
 *
 * Copyright (c) 1994
 * Hewlett-Packard Company
 *
 * Copyright (c) 1996,1997
 * Silicon Graphics Computer Systems, Inc.
 *
 * Copyright (c) 1997
 * Moscow Center for SPARC Technology
 *
 * Copyright (c) 1999 
 * Boris Fomitchev
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You may contact the author at will_mason@users.sourceforge.net.
 */

#import "HashSet.h"
#import "HashTable.h"
#import "ContainerCoder.h"
#import "Constants.h"
#import <Foundation/NSCoder.h>

@implementation OLHashSet

+ (id) hashSet
{
    OL_BEGIN_AUTO_CTOR(OLHashSet)
        init
    OL_END_AUTO_CTOR;
}

+ (id) hashSetFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last
{
    OL_BEGIN_AUTO_CTOR(OLHashSet)
        initFrom: first to: last
    OL_END_AUTO_CTOR;
}

+ (id) hashSetWithHashSet: (OLHashSet*)right
{
    OL_BEGIN_AUTO_CTOR(OLHashSet)
        initWithHashSet: right
    OL_END_AUTO_CTOR;
}

- (id) init
{
    return [self initWithTableSize: 100];
}

- (id) initFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last
{
    [self init];
    [self insertFrom: first to: last];
    return self;
}

- (id) initFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last tableSize: (unsigned)size
{
    [self initWithTableSize: size];
    [self insertFrom: first to: last];
    return self;
}

- (id) initFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last tableSize: (unsigned)size keyEqual: (NSObject<OLBoolBinaryFunction>*)eq
{
    [self initWithTableSize: size keyEqual: eq];
    [self insertFrom: first to: last];
    return self;
}

- (id) initFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last tableSize: (unsigned)size keyEqual: (NSObject<OLBoolBinaryFunction>*)eq allocator: (OLAllocator*)alloc
{
    [self initWithTableSize: size keyEqual: eq allocator: alloc];
    [self insertFrom: first to: last];
    return self;
}

- (id) initWithCoder: (NSCoder*)decoder
{
    OLAllocator* alloc = [OLContainerCoder decodeAllocatorWithCoder: decoder];
    NSObject<OLBoolBinaryFunction>* eq;
    id* array;
    OLArrayIterator* first;
    OLArrayIterator* last;
    unsigned count;
    unsigned tableSize;

    [super init];
    if ([decoder respondsToSelector: @selector(allowsKeyedCoding)] &&
        [decoder allowsKeyedCoding])
    {
        eq = [decoder decodeObjectForKey: KEYEQUAL_KEY];
        tableSize = [decoder decodeIntForKey: TABLE_SIZE_KEY];
    }
    else
    {
        eq = [decoder decodeObject];
        [decoder decodeValueOfObjCType: @encode(unsigned) at: &tableSize];
    }
    table = [[alloc allocateObject: [OLHashTable class]]
        initWithSize: tableSize equalFunc: eq allocator: alloc];
    array = [OLContainerCoder decodeWithCoder: decoder allocator: alloc returnedCount: &count];
    first = [[alloc allocateObject: [OLArrayIterator class]]
        initWithPointer: array];
    last = [[alloc allocateObject: [OLArrayIterator class]]
        initWithPointer: array + count];
    [self insertFrom: first to: last];
    [alloc deallocateBytes: array];
    [alloc deallocateObject: first];
    [alloc deallocateObject: last];
    return self;
}

- (id) initWithHashSet: (OLHashSet*)right
{
    [super init];
    table = [[[right allocator] allocateObject: [OLHashTable class]]
        initWithHashTable: right->table];
    return self;
}

- (id) initWithTableSize: (unsigned)size
{
    OLAllocator* alloc = [OLAllocator defaultAllocator];
    OLEqualTo* eq = [[alloc allocateObject: [OLEqualTo class]] init];

    [self initWithTableSize: size keyEqual: eq allocator: alloc];
    [alloc deallocateObject: eq];
    return self;
}

- (id) initWithTableSize: (unsigned)size keyEqual: (NSObject<OLBoolBinaryFunction>*)eq
{
    return [self initWithTableSize: size keyEqual: eq allocator: [OLAllocator defaultAllocator]];
}

- (id) initWithTableSize: (unsigned)size keyEqual: (NSObject<OLBoolBinaryFunction>*)eq allocator: (OLAllocator*)alloc
{
    [super init];
    table = [[alloc allocateObject: [OLHashTable class]]
        initWithSize: size equalFunc: eq allocator: alloc];
    return self;
}


- (void) dealloc
{
    [[table allocator] deallocateObject: table];
    [super dealloc];
}

- (OLHashIterator*) begin
{
    return [table begin];
}

- (void) clear
{
    [table clear];
}

- (id) copyWithAllocator: (OLAllocator*)alloc
{
    return [[alloc allocateObject: [OLHashSet class]]
        initWithHashSet: self];
}

- (id) copyWithZone: (NSZone*)zone
{
    return [[OLHashSet allocWithZone: zone] initWithHashSet: self];
}

- (unsigned) count: (id)value
{
    return [table count: value];
}

- (BOOL) empty
{
    return [table empty];
}

- (void) encodeWithCoder: (NSCoder*)encoder
{
    OLHashIterator* first = [table beginImpl];
    unsigned tableSize = [table tableSize];

    [OLContainerCoder encodeAllocator: [table allocator] withCoder: encoder];
    if ([encoder respondsToSelector: @selector(allowsKeyedCoding)] &&
        [encoder allowsKeyedCoding])
    {
        [encoder encodeObject: [table keyEqual] forKey: KEYEQUAL_KEY];
        [encoder encodeInt: tableSize forKey: TABLE_SIZE_KEY];
    }
    else
    {
        [encoder encodeObject: [table keyEqual]];
        [encoder encodeValueOfObjCType: @encode(unsigned) at: &tableSize];
    }
    [OLContainerCoder encodeFrom: first count: [table size] withCoder: encoder
        allocator: [table allocator]];
    [[table allocator] deallocateObject: first];
}

- (OLHashIterator*) end
{
    return [table end];
}

- (OLPair*) equalRange: (id)value
{
    return [table equalRange: value];
}

- (void) erase: (OLHashIterator*)where
{
    [table erase: where];
}

- (void) eraseFrom: (OLHashIterator*)first to: (OLHashIterator*)last
{
    [table eraseFrom: first to: last];
}

- (unsigned) eraseValue: (id)value
{
    return [table eraseKey: value];
}

- (OLHashIterator*) find: (id)object
{
    return [table find: object];
}

- (OLAllocator*) allocator
{
    return [table allocator];
}

- (id) insert: (id)object
{
    return [table insertUnique: object];
}

- (void) insertFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last
{
    [table insertUniqueFrom: first to: last];
}

- (BOOL) isEqual: (id)object
{
    return [object isKindOfClass: [OLHashSet class]] &&
           [((OLHashSet*)object)->table isEqualUnique: table];
}

- (NSObject<OLBoolBinaryFunction>*) keyEqual
{
    return [table keyEqual];
}

- (unsigned) maxSize
{
    return [table maxSize];
}

- (unsigned) size
{
    return [table size];
}

- (void) swap: (OLHashSet*)right
{
    [table swap: right->table];
}

@end

@implementation OLHashMultiSet

+ (id) hashMultiSet
{
    OL_BEGIN_AUTO_CTOR(OLHashMultiSet)
        init
    OL_END_AUTO_CTOR;
}

+ (id) hashMultiSetFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last
{
    OL_BEGIN_AUTO_CTOR(OLHashMultiSet)
        initFrom: first to: last
    OL_END_AUTO_CTOR;
}

+ (id) hashMultiSetWithHashSet: (OLHashSet*)right
{
    OL_BEGIN_AUTO_CTOR(OLHashMultiSet)
        initWithHashSet: right
    OL_END_AUTO_CTOR;
}

- (id) insert: (id)object
{
    return [table insertEqual: object];
}

- (void) insertFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last
{
    [table insertEqualFrom: first to: last];
}

- (BOOL) isEqual: (id)object
{
    return [object isKindOfClass: [OLHashMultiSet class]] &&
           [((OLHashMultiSet*)object)->table isEqualNonUnique: table];
}

@end
