//
// $Id: Map.m,v 1.15 2005/05/18 06:46:53 will_mason Exp $
//
// vi: set ft=objc:

/*
 * ObjectiveLib - a library of containers and algorithms for Objective-C
 *
 * Copyright (c) 2004
 * Will Mason
 *
 * Portions:
 *
 * Copyright (c) 1994
 * Hewlett-Packard Company
 *
 * Copyright (c) 1996,1997
 * Silicon Graphics Computer Systems, Inc.
 *
 * Copyright (c) 1997
 * Moscow Center for SPARC Technology
 *
 * Copyright (c) 1999 
 * Boris Fomitchev
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * You may contact the author at will_mason@users.sourceforge.net.
 */

#import "Map.h"
#import "Tree.h"
#import "ContainerCoder.h"
#import "Constants.h"
#import <Foundation/NSCoder.h>

@implementation OLMap

+ (id) map
{
    OL_BEGIN_AUTO_CTOR(OLMap)
        init
    OL_END_AUTO_CTOR;
}

+ (id) mapFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last
{
    OL_BEGIN_AUTO_CTOR(OLMap)
        initFrom: first to: last
    OL_END_AUTO_CTOR;
}

+ (id) mapWithCompare: (OLCodableFunctor<OLBoolBinaryFunction>*)comp
{
    OL_BEGIN_AUTO_CTOR(OLMap)
        initWithCompare: comp
    OL_END_AUTO_CTOR;
}

+ (id) mapWithMap: (OLMap*)right
{
    OL_BEGIN_AUTO_CTOR(OLMap)
        initWithMap: right
    OL_END_AUTO_CTOR;
}

- (id) init
{
    [super init];
    tree = [[[OLAllocator defaultAllocator] allocateObject: [OLTreeMap class]]
        init];
    return self;
}

- (id) initFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last
{
    [self init];
    [self insertFrom: first to: last];
    return self;
}

- (id) initFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last compare: (OLCodableFunctor<OLBoolBinaryFunction>*)comp
{
    [self initWithCompare: comp];
    [self insertFrom: first to: last];
    return self;
}

- (id) initFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last compare: (OLCodableFunctor<OLBoolBinaryFunction>*)comp allocator: (OLAllocator*)alloc
{
    [self initWithCompare: comp allocator: alloc];
    [self insertFrom: first to: last];
    return self;
}

- (id) initWithCoder: (NSCoder*)decoder
{
    OLAllocator* alloc = [OLContainerCoder decodeAllocatorWithCoder: decoder];
    OLCodableFunctor<OLBoolBinaryFunction>* comp;
    id* array;
    OLArrayIterator* first;
    OLArrayIterator* last;
    unsigned count;

    if ([decoder respondsToSelector: @selector(allowsKeyedCoding)] &&
        [decoder allowsKeyedCoding])
    {
        comp = [decoder decodeObjectForKey: COMPARATOR_KEY];
    }
    else
    {
        comp = [decoder decodeObject];
    }
    array = [OLContainerCoder decodeWithCoder: decoder allocator: alloc returnedCount: &count];
    first = [[alloc allocateObject: [OLArrayIterator class]]
        initWithPointer: array];
    last = [[alloc allocateObject: [OLArrayIterator class]]
        initWithPointer: array + count];
    [self initFrom: first to: last compare: comp allocator: alloc];
    [alloc deallocateBytes: array];
    [alloc deallocateObject: first];
    [alloc deallocateObject: last];
    return self;
}

- (id) initWithCompare: (OLCodableFunctor<OLBoolBinaryFunction>*)comp
{
    [super init];
    tree = [(OLTree*)[[OLAllocator defaultAllocator] allocateObject: [OLTreeMap class]]
        initWithCompare: comp];
    return self;
}

- (id) initWithCompare: (OLCodableFunctor<OLBoolBinaryFunction>*)comp allocator: (OLAllocator*)alloc
{
    [super init];
    tree = [(OLTree*)[alloc allocateObject: [OLTreeMap class]]
        initWithCompare: comp allocator: alloc];
    return self;
}

- (id) initWithMap: (OLMap*)right
{
    [super init];
    tree = [[[right allocator] allocateObject: [OLTreeMap class]]
        initWithTree: right->tree];
    return self;
}

- (void) dealloc
{
    [[tree allocator] deallocateObject: tree];
    [super dealloc];
}

- (void) assignKey: (id)key value: (id)value
{
    [tree assignUniqueKey: key value: value];
}

- (OLAssociativeIterator*) begin
{
    return [tree begin];
}

- (void) clear
{
    [tree clear];
}

- (id) copyWithAllocator: (OLAllocator*)alloc
{
    return [[alloc allocateObject: [OLMap class]] initWithMap: self];
}

- (id) copyWithZone: (NSZone*)zone
{
    return [[OLMap allocWithZone: zone] initWithMap: self];
}

- (unsigned) count: (id)key
{
    return [tree count: key];
}

- (BOOL) empty
{
    return [tree empty];
}

- (void) encodeWithCoder: (NSCoder*)encoder
{
    OLAssociativeIterator* first = [tree beginImpl];

    [OLContainerCoder encodeAllocator: [tree allocator] withCoder: encoder];
    if ([encoder respondsToSelector: @selector(allowsKeyedCoding)] &&
        [encoder allowsKeyedCoding])
    {
        [encoder encodeObject: [tree keyComp] forKey: COMPARATOR_KEY];
    }
    else
    {
        [encoder encodeObject: [tree keyComp]];
    }
    [OLContainerCoder encodeFrom: first count: [tree size] withCoder: encoder
        allocator: [tree allocator]];
    [[tree allocator] deallocateObject: first];
}

- (OLAssociativeIterator*) end
{
    return [tree end];
}

- (OLPair*) equalRange: (id)key
{
    return [tree equalRange: key];
}

- (void) erase: (OLAssociativeIterator*)where
{
    [tree erase: where];
}

- (void) eraseFrom: (OLAssociativeIterator*)first to: (OLAssociativeIterator*)last
{
    [tree eraseFrom: first to: last];
}

- (unsigned) eraseKey: (id)key
{
    return [tree eraseKey: key];
}

- (OLAssociativeIterator*) find: (id)key
{
    return [tree find: key];
}

- (OLAllocator*) allocator
{
    return [tree allocator];
}

- (id) insert: (OLPair*)keyValue
{
    return [tree insertUnique: keyValue];
}

- (OLAssociativeIterator*) insertAt: (OLAssociativeIterator*)where value: (OLPair*)keyVal
{
    return [tree insertUniqueAt: where value: keyVal];
}

- (void) insertFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last
{
    [tree insertUniqueFrom: first to: last];
}

- (id) insertKey: (id)key value: (id)val
{
    OLPair* p = [[[tree allocator] allocateObject: [OLPair class]]
        initWithFirst: key second: val];
    id r = [self insert: p];

    [[tree allocator] deallocateObject: p];
    return r;
}

- (BOOL) isEqual: (id)object
{
    return [object isKindOfClass: [OLMap class]] &&
           [tree isEqualUnique: ((OLMap*)object)->tree];
}

- (OLCodableFunctor<OLBoolBinaryFunction>*) keyComp
{
    return [tree keyComp];
}

- (OLAssociativeIterator*) lowerBound: (id)key
{
    return [tree lowerBound: key];
}

- (unsigned) maxSize
{
    return [tree maxSize];
}

- (OLReverseBidiIterator*) rbegin
{
    return [tree rbegin];
}

- (OLReverseBidiIterator*) rend
{
    return [tree rend];
}

- (unsigned) size
{
    return [tree size];
}

- (void) swap: (OLMap*)right
{
    [tree swap: right->tree];
}

- (OLAssociativeIterator*) upperBound: (id)key
{
    return [tree upperBound: key];
}

- (OLCodableFunctor<OLBoolBinaryFunction>*) valueComp
{
    return [tree keyComp];
}

- (id) valueForKey: (id)key
{
    return [tree findValue: key];
}

@end

@implementation OLMultiMap

+ (id) multiMap
{
    OL_BEGIN_AUTO_CTOR(OLMultiMap)
        init
    OL_END_AUTO_CTOR;
}

+ (id) multiMapFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last
{
    OL_BEGIN_AUTO_CTOR(OLMultiMap)
        initFrom: first to: last
    OL_END_AUTO_CTOR;
}

+ (id) multiMapWithCompare: (OLCodableFunctor<OLBoolBinaryFunction>*)comp
{
    OL_BEGIN_AUTO_CTOR(OLMultiMap)
        initWithCompare: comp
    OL_END_AUTO_CTOR;
}

+ (id) multiMapWithMap: (OLMap*)right
{
    OL_BEGIN_AUTO_CTOR(OLMultiMap)
        initWithMap: right
    OL_END_AUTO_CTOR;
}

- (void) assignKey: (id)key value: (id)value
{
    [tree assignEqualKey: key value: value];
}

- (id) insert: (OLPair*)keyValue
{
    return [tree insertEqual: keyValue];
}

- (OLAssociativeIterator*) insertAt: (OLAssociativeIterator*)where value: (OLPair*)keyVal
{
    return [tree insertEqualAt: where value: keyVal];
}

- (void) insertFrom: (OLForwardIterator*)first to: (OLForwardIterator*)last
{
    [tree insertEqualFrom: first to: last];
}

- (BOOL) isEqual: (id)object
{
    return [object isKindOfClass: [OLMultiMap class]] &&
           [tree isEqualNonUnique: ((OLMultiMap*)object)->tree];
}

- (id) valueForKey: (id)key
{
    return nil;
}

@end
